@@ -2,6 +2,7 @@ import numpy as np
 import random
 import json
 from matplotlib import pyplot as plt
+from base import activation_function, loss_function
 
 class FNN():
     """
@@ -48,6 +49,9 @@ class FNN():
      
      Methods
      -------
+     weight_initializer(name="random"):
+         Initializes weights and biases.
+
      init_params(sizes, epochs):
          Initializes parameters in the NN.
      
@@ -131,6 +135,40 @@ class FNN():
         self.loss_fn = loss_fn
         self.epoch_list = list()
         self.accuracy = list()
+        
+    def weight_initializer(self, name="random"):
+        """
+        Initializes weights and biases
+        
+        Parameters
+        ----------
+            
+        name: str
+            Type of weight initialization.
+            Options:
+                random ( Gauss distro mean 0, std 1 )
+                xavier ( n^2 = 1 / n )
+                he ( n^2 = 2 / n )
+        
+        Returns
+        -------
+        None
+        """
+        
+        if name == "random":
+            self.weights = [np.random.randn(y, x) 
+                          for x, y in zip(self.sizes[:-1], self.sizes[1:])]
+            self.biases = [np.random.randn(y, 1) for y in self.sizes[1:]]
+         
+        elif name == "xavier":
+            self.weights = [np.random.randn(y, x)/np.sqrt(x) 
+                          for x, y in zip(self.sizes[:-1], self.sizes[1:])]
+            self.biases = [np.random.randn(y, 1) for y in self.sizes[1:]]
+        
+        elif name == "he":
+            self.weights = [np.random.randn(y, x)*np.sqrt(2/x)
+                          for x, y in zip(self.sizes[:-1], self.sizes[1:])]
+            self.biases = [np.random.randn(y, 1) for y in self.sizes[1:]]
 
     def init_params(self, sizes, epochs, weight_init_type=None):
         """
@@ -154,7 +192,7 @@ class FNN():
         """
         
         self.n_layers = len(sizes)
-        if weight_init_type: weight_initializer(self, weight_init_type)
+        if weight_init_type: self.weight_initializer(weight_init_type)
         self.prev_update_w = [np.zeros(w.shape) for w in self.weights]
         self.prev_update_b = [np.zeros(b.shape) for b in self.biases]
         self.epoch_list = np.arange(0, epochs)
@@ -739,146 +777,3 @@ class FNN():
         self.biases = [np.array(b) for b in config["biases"]]
         self.activation_types = config["activation_fns"]
         self.loss_fn = config["loss"]
-
-def weight_initializer(NN, name="random"):
-    """
-    Initializes weights and biases
-    
-    Parameters
-    ----------
-    NN: Neural net object
-        contains sizes, weights, biases
-        
-    name: str
-        Type of weight initialization.
-        Options:
-            random ( Gauss distro mean 0, std 1 )
-            xavier ( n^2 = 1 / n )
-            he ( n^2 = 2 / n )
-    
-    Returns
-    -------
-    NN: Neural net object
-    """
-    
-    if name == "random":
-        NN.weights = [np.random.randn(y, x) 
-                      for x, y in zip(NN.sizes[:-1], NN.sizes[1:])]
-        NN.biases = [np.random.randn(y, 1) for y in NN.sizes[1:]]
-     
-    elif name == "xavier":
-        NN.weights = [np.random.randn(y, x)/np.sqrt(x) 
-                      for x, y in zip(NN.sizes[:-1], NN.sizes[1:])]
-        NN.biases = [np.random.randn(y, 1) for y in NN.sizes[1:]]
-    
-    elif name == "he":
-        NN.weights = [np.random.randn(y, x)*np.sqrt(2/x)
-                      for x, y in zip(NN.sizes[:-1], NN.sizes[1:])]
-        NN.biases = [np.random.randn(y, 1) for y in NN.sizes[1:]]
-        
-    return NN
-
-def activation_function(name, input, derivative=False):
-    """
-    Computes the activation function and its derivative.
-    
-    Parameters
-    ----------
-    name: str
-        Activation function name.
-          Options:
-              identity
-              sigmoid
-              softmax
-              tanh
-              relu  
-    
-    input: int/float/list/array
-    
-    derivative: bool
-        If true, returns the derivative of loss.
-        Default: False
-    
-    Returns
-    -------
-    Numpy array or list
-    """
-    
-    if name == "identity":
-        if derivative:
-            return np.ones_like(input)
-        else:
-            return input
-    elif name == "sigmoid":
-        if derivative:
-            out = activation_function(name, input)
-            return out * ( 1 - out )
-        else:
-            return 1 / (1 + np.exp(-input))
-    elif name == "softmax":
-        if derivative:
-            out = activation_function(name, input)
-            return out * (1 - out)
-        else:
-            e_x = np.exp(input - np.max(input))
-            return e_x / np.sum(e_x, axis=1, keepdims=True)
-    elif name == "tanh":
-        if derivative:
-            out = activation_function(name, input)
-            return 1 - np.square(out)
-        else:
-            return 2 / (1 + np.exp(-2*input)) - 1
-    elif name == "relu":
-        if derivative:
-            return (input > 0) * 1
-        else:
-            return np.maximum(0, input)
-      
-def loss_function(name, y, y_hat, derivative=False):
-    """
-    Computes the loss and its derivative
-    
-    Parameters
-    ----------
-    name: str
-        Type of loss function.
-        Options:
-            mse ( Mean squared error )
-            ce ( Cross entropy ) 
-    
-    y: list 
-        numpy array ( target )
-    
-    y_hat: list
-        numpy array ( output )
-    
-    derivative: bool
-        If True, returns the derivative of loss.
-        Default: False
-    
-    Returns
-    -------
-    numpy array
-    """
-    
-    # y - target, y_hat - output
-    # Mean Squared Error
-    if name == "mse":
-        if derivative:
-            return (y_hat - y)
-        else:
-            return np.mean((y - y_hat)**2)
-    # Log-likelihood
-    elif name == "ll":
-        if derivative:
-            return - (1 / y_hat)
-        else:
-            return -1 * np.log(y_hat)
-    # y - target prob distro, y_hat - output prob distro
-    # Cross Entropy
-    elif name == "ce":
-        if derivative:
-            # if activation fn is sigmoid/softmax
-            return (y_hat - y)    
-        else:
-            return np.sum(np.nan_to_num(-y*np.log(y_hat)-(1-y)*np.log(1-y_hat)))
\ No newline at end of file
